@@ -8,6 +8,7 @@
 
 #else
 
+#define R_PosInf INFINITY
 #define R_NegInf -INFINITY
 
 #include <stdint.h>
@@ -85,7 +86,7 @@ protected:
         virtual void setDouble (void *ptr, const double value) const {}
         virtual void setInt (void *ptr, const int value) const {}
         virtual void setRgb (void *ptr, const rgba32_t value) const {}
-        virtual void minmax (void *ptr, const size_t length, double *min, double *max) const { *min = 0.0; *max = 0.0; }
+        virtual void minmax (void *ptr, const size_t length, double *min, double *max, const bool dropNaN = true) const { *min = R_PosInf; *max = R_NegInf; }
     };
     
     /**
@@ -99,15 +100,33 @@ protected:
         complex128_t getComplex (void *ptr) const { return complex128_t(static_cast<double>(*static_cast<Type*>(ptr)), 0.0); }
         double getDouble (void *ptr) const { return static_cast<double>(*static_cast<Type*>(ptr)); }
         int getInt (void *ptr) const { return static_cast<int>(*static_cast<Type*>(ptr)); }
-        void setComplex (void *ptr, const complex128_t value) const
-        {
-            *(static_cast<Type*>(ptr)) = Type(value.real());
-            *(static_cast<Type*>(ptr) + 1) = Type(0);
-        }
         void setDouble (void *ptr, const double value) const { *(static_cast<Type*>(ptr)) = Type(value); }
         void setInt (void *ptr, const int value) const { *(static_cast<Type*>(ptr)) = Type(value); }
-        void minmax (void *ptr, const size_t length, double *min, double *max) const;
+        void minmax (void *ptr, const size_t length, double *min, double *max, const bool dropNaN = true) const;
     };
+
+#ifdef USING_R
+    template <bool alpha>
+    struct ConcreteTypeHandler<int,alpha> : public TypeHandler
+    {
+        size_t size () const { return (sizeof(int)); }
+        bool hasNaN () const { return true; }
+        complex128_t getComplex (void *ptr) const {
+            int value = *static_cast<int*>(ptr);
+            return complex128_t(value == NA_INTEGER ? NA_REAL : static_cast<double>(value), 0.0);
+        }
+        double getDouble (void *ptr) const {
+            int value = *static_cast<int*>(ptr);
+            return (value == NA_INTEGER ? NA_REAL : static_cast<double>(value));
+        }
+        int getInt (void *ptr) const { return *static_cast<int*>(ptr); }
+        void setDouble (void *ptr, const double value) const {
+            *(static_cast<int*>(ptr)) = ISNA(value) ? NA_INTEGER : int(value);
+        }
+        void setInt (void *ptr, const int value) const { *(static_cast<int*>(ptr)) = value; }
+        void minmax (void *ptr, const size_t length, double *min, double *max, const bool dropNaN = true) const;
+    };
+#endif
     
     template <typename ElementType>
     struct ConcreteTypeHandler<std::complex<ElementType>,false> : public TypeHandler
@@ -131,7 +150,10 @@ protected:
         void setComplex (void *ptr, const complex128_t value) const { setNative(ptr, std::complex<ElementType>(value)); }
         void setDouble (void *ptr, const double value) const { setNative(ptr, std::complex<ElementType>(value, 0.0)); }
         void setInt (void *ptr, const int value) const { setNative(ptr, std::complex<ElementType>(static_cast<ElementType>(value), 0.0)); }
-        void minmax (void *ptr, const size_t length, double *min, double *max) const;
+        void minmax (void *ptr, const size_t length, double *min, double *max, const bool dropNaN = true)
+        {
+            throw std::runtime_error("Complex values don't have a simple ordering, so maxima and minima are ill-defined");
+        };
     };
     
     template <bool alpha>
@@ -157,7 +179,7 @@ protected:
             unsigned char *target = static_cast<unsigned char *>(ptr);
             std::copy(value.value.bytes, value.value.bytes + (alpha ? 4 : 3), target);
         }
-        void minmax (void *ptr, const size_t length, double *min, double *max) const { *min = 0.0; *max = 255.0; }
+        void minmax (void *ptr, const size_t length, double *min, double *max, const bool dropNaN = true) const { *min = 0.0; *max = 255.0; }
     };
     
     /**
@@ -603,27 +625,22 @@ public:
      *   datatype is unknown or the data is empty
      * @param max Pointer to the maximum value (output parameter). Will be set to zero if the
      *   datatype is unknown or the data is empty
+     * @param dropNaN Boolean value indicating whether missing/NaN values should be ignored.
+     *   This is analogous to the \c na.rm argument in R. If \c false, and a NaN value is
+     *   encountered, then the results will both be NaN
     **/
-    void minmax (double *min, double *max) const
+    void minmax (double *min, double *max, const bool dropNaN = true) const
     {
         if (handler == NULL)
         {
-            *min = 0.0;
-            *max = 0.0;
+            *min = R_PosInf;
+            *max = R_NegInf;
         }
         else
-            handler->minmax(dataPtr, _length, min, max);
+            handler->minmax(dataPtr, _length, min, max, dropNaN);
     }
 };
 
-
-// R provides an NaN (NA) value for integers
-#ifdef USING_R
-template <>
-inline bool NiftiImageData::ConcreteTypeHandler<int>::hasNaN () const { return true; }
-#endif
-
-
 /**
  * A simple object-oriented wrapper around a fixed-length array.
  * @author Jon Clayden (<code@clayden.org>)
