@@ -839,6 +839,104 @@ BEGIN_RCPP
 END_RCPP
 }
 
+inline static double naPropagatingPlus (const double &x, const double &y)
+{
+    return (ISNA(x) || ISNA(y)) ? NA_REAL : x + y;
+}
+
+inline static double naPropagatingTimes (const double &x, const double &y)
+{
+    return (ISNA(x) || ISNA(y)) ? NA_REAL : x * y;
+}
+
+inline static double naDiscardingPlus (const double &x, const double &y)
+{
+    return (ISNA(x) ? 0.0 : x) + (ISNA(y) ? 0.0 : y);
+}
+
+inline static double naDiscardingTimes (const double &x, const double &y)
+{
+    return (ISNA(x) ? 1.0 : x) * (ISNA(y) ? 1.0 : y);
+}
+
+RcppExport SEXP summariseImage (SEXP _image, SEXP _generic, SEXP _na_rm)
+{
+BEGIN_RCPP
+    const NiftiImage image(_image, true, true);
+    const NiftiImageData data = image.data();
+    const std::string generic = as<std::string>(_generic);
+    const bool dropNAs = as<bool>(_na_rm);
+    
+    if (generic == "any" || generic == "all")
+        Rf_error("Images do not have logical type, so \"any\" and \"all\" generics are invalid");
+    else if (data.isComplex() && (generic == "min" || generic == "max" || generic == "range"))
+        Rf_error("Complex values don't have a simple ordering, so maxima and minima are ill-defined");
+    
+    if (data.isEmpty() || data.length() == 0)
+    {
+        Rf_warning("Taking summary value from an empty image");
+        if (generic == "max")           return Rf_ScalarReal(R_NegInf);
+        else if (generic == "min")      return Rf_ScalarReal(R_PosInf);
+        else if (generic == "range")    return NumericVector::create(R_PosInf, R_NegInf);
+        else if (generic == "sum")      return Rf_ScalarReal(0.0);
+        else if (generic == "prod")     return Rf_ScalarReal(1.0);
+        else                            Rf_error("Unexpected generic name: \"%s\"", generic.c_str());
+    }
+    else if (data.isComplex())
+    {
+        complex128_t result(generic == "prod" ? 1.0 : 0.0, 0.0);
+        for (NiftiImageData::Iterator it=data.begin(); it!=data.end(); it++)
+        {
+            complex128_t value = *it;
+            if (RNifti::internal::isNA(value.real()) || RNifti::internal::isNA(value.imag()))
+            {
+                if (!dropNAs)
+                {
+                    if (generic == "range")
+                        return ComplexVector::create(complexNA(), complexNA());
+                    else
+                        return Rf_ScalarComplex(complexNA());
+                }
+            }
+            else if (generic == "sum")
+                result += value;
+            else if (generic == "prod")
+                result *= value;
+            else
+                Rf_error("Unexpected generic name: \"%s\"", generic.c_str());
+        }
+        return wrap(result);
+    }
+    else if (generic == "sum")
+    {
+        double result;
+        if (dropNAs)
+            result = std::accumulate(data.begin(), data.end(), 0.0, naDiscardingPlus);
+        else
+            result = std::accumulate(data.begin(), data.end(), 0.0, naPropagatingPlus);
+        return Rf_ScalarReal(result);
+    }
+    else if (generic == "prod")
+    {
+        double result;
+        if (dropNAs)
+            result = std::accumulate(data.begin(), data.end(), 1.0, naDiscardingTimes);
+        else
+            result = std::accumulate(data.begin(), data.end(), 1.0, naPropagatingTimes);
+        return Rf_ScalarReal(result);
+    }
+    else
+    {
+        double min, max;
+        data.minmax(&min, &max, dropNAs);
+        if (generic == "max")           return Rf_ScalarReal(max);
+        else if (generic == "min")      return Rf_ScalarReal(min);
+        else if (generic == "range")    return NumericVector::create(min, max);
+        else                            Rf_error("Unexpected generic name: \"%s\"", generic.c_str());
+    }
+END_RCPP
+}
+
 RcppExport SEXP rescaleImage (SEXP _image, SEXP _scales)
 {
 BEGIN_RCPP
@@ -956,6 +1054,7 @@ R_CallMethodDef callMethods[] = {
     { "hasData",        (DL_FUNC) &hasData,         1 },
     { "indexVector",    (DL_FUNC) &indexVector,     2 },
     { "indexList",      (DL_FUNC) &indexList,       2 },
+    { "summariseImage", (DL_FUNC) &summariseImage,  3 },
     { "rescaleImage",   (DL_FUNC) &rescaleImage,    2 },
     { "pointerToArray", (DL_FUNC) &pointerToArray,  1 },
     { "unwrapPointer",  (DL_FUNC) &unwrapPointer,   2 },
